import { BasicColumn, FormSchema } from '@/components/Table';
import { h } from 'vue';
import {Switch, Tag} from 'ant-design-vue';
import Icon from '@/components/Icon/Icon.vue';
import {useMessage} from "@/hooks/web/useMessage";
import {setMenuStatus} from "@/api/system/menu/menu";

export const columns: BasicColumn[] = [
  {
    title: '菜单名称',
    dataIndex: 'menuName',
    width: 200,
    align: 'left',
  },
  {
    title: '图标',
    dataIndex: 'icon',
    width: 50,
    customRender: ({ record }) => {
      return h(Icon, { icon: record.icon });
    },
  },
  {
    title: '排序',
    dataIndex: 'orderNum',
    width: 50,
  },
  {
    title: '权限标识',
    dataIndex: 'perms',
    width: 180,
  },
  {
    title: '组件路径',
    dataIndex: 'path',
    width: 200,
  },
  {
    title: '状态',
    dataIndex: 'flag',
    width: 120,
    customRender: ({ record }) => {
      if (!Reflect.has(record, 'pendingStatus')) {
        record.pendingStatus = false;
      }
      return h(Switch, {
        checked: record.flag === '1',
        loading: record.pendingStatus,
        onChange(checked: CheckedType) {
          record.pendingStatus = true;
          const newStatus = checked ? '1' : '0';
          const { createMessage } = useMessage();
          const params = {
            businessId: record.businessId,
            flag: newStatus,
            menuName: record.menuName,
            parentId: record.parentId,
            menuType: record.menuType,
            jumpType: record.jumpType,
            orderNum: record.orderNum,
            isFrame: record.isFrame,
            icon: record.icon,
            visible: record.visible,
            isProcess: record.isProcess,
            path: record.path,
            perms: record.perms,
            component: record.component,
          }
          setMenuStatus(params)
            .then(() => {
              record.flag = newStatus;
              const text = record.flag === '1' ? '启用' : '停用'
              createMessage.success( text + `成功`);
            })
            .catch(() => {
              // createMessage.error('操作失败');
            })
            .finally(() => {
              record.pendingStatus = false;
            });
        },
      });
    },
  },
  {
    title: '创建时间',
    dataIndex: 'createDate',
    width: 180,
  },
];

const isDir = (menuType: string) => menuType === 'M';
const isMenu = (menuType: string) => menuType === 'C';
const isButton = (menuType: string) => menuType === 'F';

export const searchFormSchema: FormSchema[] = [
  {
    field: 'menuName',
    label: '菜单名称',
    component: 'Input',
    colProps: { span: 7 },
  },
];

export const formSchema: FormSchema[] = [
  {
    field: 'menuType',
    label: '菜单类型',
    component: 'RadioButtonGroup',
    defaultValue: 'M',
    componentProps: {
      options: [
        { label: '目录', value: 'M' },
        { label: '菜单', value: 'C' },
        { label: '按钮', value: 'F' },
      ],
    },
    colProps: { lg: 24, md: 24 },
  },
  {
    field: 'jumpType',
    label: '跳转类型',
    component: 'RadioButtonGroup',
    componentProps: {
      options: [
        { label: '正常路由跳转', value: 'R' },
        { label: '新页面无菜单', value: 'Y' },
        { label: '新页面有菜单', value: 'G' },
      ],
    },
    colProps: { lg: 24, md: 24 },
    ifShow: ({ values }) => isMenu(values.menuType),
  },
  {
    field: 'menuName',
    label: '菜单名称',
    component: 'Input',
    rules: [
      {
        required: true,
        message: '请输入菜单名称',
      },
    ],
  },
  {
    field: 'parentId',
    label: '上级菜单',
    component: 'TreeSelect',
    rules: [
      {
        required: true,
        message: '请选择上级菜单',
      },
    ],
    componentProps: {
      fieldNames: {
        label: 'menuName',
        value: 'businessId',
      },
      getPopupContainer: () => document.body,
    },
  },
  {
    field: 'orderNum',
    label: '菜单排序',
    component: 'InputNumber',
    rules: [
      {
        required: true,
        message: '请输入菜单排序',
      },
    ],
    componentProps: {
      min: 0, // 最小值
      max: 9999, // 最大值（根据需要调整）
      step: 1, // 每次增减的步长
      placeholder: '请输入排序号', // 占位符
      style: { width: '100%' }, // 样式（可选）
    },
  },
  {
    field: 'icon',
    label: '菜单图标',
    component: 'IconPicker',
    ifShow: ({ values }) => !isButton(values.menuType),
  },
  {
    field: 'component',
    label: '组件路径',
    component: 'Input',
    ifShow: ({ values }) => isMenu(values.menuType),
  },
  {
    field: 'perms',
    label: '权限标识',
    component: 'Input',
    ifShow: ({ values }) => !isDir(values.menuType),
  },
  {
    field: 'flag',
    label: '菜单状态',
    component: 'RadioButtonGroup',
    defaultValue: '0',
    componentProps: {
      options: [
        { label: '启用', value: '1' },
        { label: '停用', value: '0' },
      ],
    },
    ifShow: ({ values }) => !isButton(values.menuType),
  },
  {
    field: 'visible',
    label: '显示状态',
    component: 'RadioButtonGroup',
    defaultValue: '0',
    componentProps: {
      options: [
        { label: '显示', value: '1' },
        { label: '隐藏', value: '0' },
      ],
    },
    ifShow: ({ values }) => !isButton(values.menuType),
  },
  {
    field: 'isFrame',
    label: '是否外链',
    component: 'RadioButtonGroup',
    defaultValue: '0',
    componentProps: {
      options: [
        { label: '是', value: '1' },
        { label: '否', value: '0' },
      ],
    },
    ifShow: ({ values }) => !isButton(values.menuType),
  },
  {
    field: 'isProcess',
    label: '是工作流',
    component: 'RadioButtonGroup',
    defaultValue: '0',
    componentProps: {
      options: [
        { label: '是', value: '1' },
        { label: '否', value: '0' },
      ],
    },
    ifShow: ({ values }) => !isButton(values.menuType),
  },
  {
    field: 'path',
    label: '路由地址',
    component: 'Input',
    rules: [
      {
        required: true,
        message: '请输入路由地址',
      },
    ],
    ifShow: ({ values }) => !isButton(values.menuType),
  },
];
