import { getAllRoleList } from '@/api/system/role/role';
import { BasicColumn, FormSchema } from '@/components/Table';
import { h } from 'vue';
import { Switch } from 'ant-design-vue';
import { useMessage } from '@/hooks/web/useMessage';
import { changeFlagApi } from '@/api/system/user/user';
// 引入开关组件
type CheckedType = boolean | string | number;
/**
 * transform mock data
 * {
 *  0: '华东分部',
 * '0-0': '华东分部-研发部'
 * '0-1': '华东分部-市场部',
 *  ...
 * }
 */
export const deptMap = (() => {
  const pDept = ['华东分部', '华南分部', '西北分部'];
  const cDept = ['研发部', '市场部', '商务部', '财务部'];

  return pDept.reduce((map, p, pIdx) => {
    map[pIdx] = p;

    cDept.forEach((c, cIndex) => (map[`${pIdx}-${cIndex}`] = `${p}-${c}`));

    return map;
  }, {});
})();

export const columns: BasicColumn[] = [
  {
    title: '账号',
    dataIndex: 'username',
    width: 120,
  },
  {
    title: '姓名',
    dataIndex: 'name',
    width: 120,
  },
  {
    title: '邮箱',
    dataIndex: 'email',
    width: 120,
  },
  {
    title: '状态',
    dataIndex: 'flag',
    width: 30,
    customRender: ({ record }) => {
      if (!Reflect.has(record, 'pendingStatus')) {
        record.pendingStatus = false;
      }
      return h(Switch, {
        checked: record.flag === '1',
        // checkedChildren: '停用',
        // unCheckedChildren: '启用',
        loading: record.pendingStatus,
        onChange(checked: CheckedType) {
          record.pendingStatus = true;
          const newStatus = checked ? '1' : '0';
          const { createMessage } = useMessage();
          const params = {
            businessId: record.businessId,
            flag: newStatus,
          };
          changeFlagApi(params)
            .then(() => {
              record.flag = newStatus;
              const text = record.flag === '1' ? '启用' : '停用';
              createMessage.success(text + `成功`);
            })
            .catch(() => {
              // createMessage.error('操作失败');
            })
            .finally(() => {
              record.pendingStatus = false;
            });
        },
      });
    },
  },
  {
    title: '创建时间',
    dataIndex: 'createDate',
    width: 140,
  },
  // {
  //   title: '所属部门',
  //   dataIndex: 'dept',
  //   customRender: ({ value }) => {
  //     return deptMap[value];
  //   },
  // },
];

export const searchFormSchema: FormSchema[] = [
  {
    field: 'username',
    label: '账号',
    component: 'Input',
    colProps: { span: 8 },
  },
  {
    field: 'flag',
    label: '状态',
    component: 'Select',
    componentProps: {
      options: [
        { label: '启用', value: '1' },
        { label: '停用', value: '0' },
      ],
    },
    colProps: { span: 8 },
  },
];
export const resetPasswordFormSchema: any[] = [
  {
    field: 'newPassword',
    label: '新密码',
    component: 'InputPassword',
    componentProps: {
      placeholder: '请输入8~16位，由字母和数字混合所组成的新密码',
    },
    required: true,
    // ifShow: false,
  },
];
const passwordCheck = (rule, value, callback) => {
  const pattern = /^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{8,16}$/;
  if (!pattern.test(value)) {
    callback(new Error('新密码必须为数字与字母的组合'));
  }
  callback();
};
export const accountFormSchema: any[] = [
  {
    field: 'username',
    label: '账号',
    component: 'Input',
    // helpMessage: ['本字段演示异步验证', '不能输入带有admin的用户名'],
    rules: [
      {
        required: true,
        message: '请输入登录名',
      },
    ],
  },
  {
    field: 'password',
    label: '密码',
    component: 'InputPassword',
    rules: [
      {
        required: true,
        message: '请输入用户密码',
      },
      { min: 8, max: 16, message: '长度在 8 到 16 个字符', trigger: 'blur' },
      {
        validator: passwordCheck,
        trigger: 'blur',
      },
    ],
  },
  {
    field: 'name',
    label: '姓名',
    component: 'Input',
  },
  {
    field: 'nickName',
    label: '昵称',
    component: 'Input',
  },
  {
    field: 'sex',
    label: '性别',
    component: 'Select',
    componentProps: {
      options: [
        { label: '男', value: '1' },
        { label: '女', value: '0' },
      ],
    },
  },
  {
    field: 'phone',
    label: '手机号码',
    component: 'Input',
    rules: [
      { required: false, message: '请输入手机号码', trigger: 'blur' },
      {
        pattern: /^1[3|4|5|6|7|8|9][0-9]\d{8}$/,
        message: '请输入正确的手机号码',
        trigger: 'blur',
      },
    ],
  },
  {
    field: 'idNumber',
    label: '身份证号',
    component: 'Input',
    rules: [
      {
        required: false,
        message: '请输入身份证号',
        trigger: 'blur',
      },
      { min: 18, max: 18, message: '长度18字符', trigger: 'blur' },
    ],
  },
  {
    field: 'email',
    label: '邮箱',
    component: 'Input',
    rules: [
      { required: false, message: '请输入邮箱地址', trigger: 'blur' },
      {
        type: 'email',
        message: '请输入正确的邮箱地址',
        trigger: ['blur', 'change'],
      },
    ],
  },
  {
    field: 'flag',
    label: '状态',
    component: 'RadioGroup',
    colProps: { lg: 24, md: 24 },
    componentProps: {
      options: [
        { label: '启用', value: '1' },
        { label: '停用', value: '0' },
      ],
    },
    required: true,
  },
  {
    field: 'deptId',
    label: '所属部门',
    component: 'TreeSelect',
    colProps: { lg: 24, md: 24 },
    componentProps: {
      fieldNames: {
        label: 'deptName',
        value: 'businessId',
      },
      getPopupContainer: () => document.body,
    },
    required: true,
  },
  {
    field: 'roleList',
    label: '角色',
    component: 'ApiSelect',
    colProps: { lg: 24, md: 24 },
    componentProps: {
      // placeholder: '公开给',
      mode: 'multiple',
      api: getAllRoleList,
      labelField: 'roleName',
      valueField: 'businessId',
      resultField: 'data',
    },
    required: true,
  },
];

export const searchDetailFormSchema: FormSchema[] = [
  {
    field: 'username',
    label: '角色名称',
    component: 'Input',
    colProps: { span: 8 },
  },
];

export const detailColumns: BasicColumn[] = [
  {
    title: '角色名称',
    dataIndex: 'roleName',
    width: 200,
  },
  {
    title: '角色描述',
    dataIndex: 'remark',
    width: 200,
  },
];
export const groupColumns: BasicColumn[] = [
  {
    title: '工作区名称',
    dataIndex: 'workspaceName',
    width: 200,
  },
  {
    title: '赋予角色',
    dataIndex: 'roleName',
    width: 200,
  },
];
